# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2022 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/

__authors__ = ["H. Payno"]
__license__ = "MIT"
__date__ = "03/05/2022"


import pytest
from tomoscan.utils import BoundingBox1D, _BoundingBox


def test_bounding_box_base():
    bb = _BoundingBox(0, 1)
    with pytest.raises(NotImplementedError):
        bb.get_overlap(None)


def test_bounding_box_1D():
    # check overlaping
    bb1 = BoundingBox1D(0.0, 1.0)
    bb2 = BoundingBox1D(0.2, 1.0)
    assert bb1.get_overlap(bb2) == BoundingBox1D(0.2, 1.0)
    assert bb2.get_overlap(bb1) == BoundingBox1D(0.2, 1.0)

    bb1 = BoundingBox1D(0.0, 1.0)
    bb2 = BoundingBox1D(0.2, 1.2)
    assert bb1.get_overlap(bb2) == BoundingBox1D(0.2, 1.0)
    assert bb2.get_overlap(bb1) == BoundingBox1D(0.2, 1.0)

    bb1 = BoundingBox1D(0.0, 1.0)
    bb2 = BoundingBox1D(0.2, 0.8)
    assert bb1.get_overlap(bb2) == BoundingBox1D(0.2, 0.8)
    assert bb2.get_overlap(bb1) == BoundingBox1D(0.2, 0.8)

    bb1 = BoundingBox1D(0.0, 1.0)
    bb2 = BoundingBox1D(1.0, 1.2)
    assert bb2.get_overlap(bb1) == BoundingBox1D(1.0, 1.0)

    # check outside
    bb1 = BoundingBox1D(0.0, 1.0)
    bb2 = BoundingBox1D(2.0, 2.2)

    assert bb2.get_overlap(bb1) is None
    assert bb1.get_overlap(bb2) is None

    # check on fully including in the other
    bb1 = BoundingBox1D(0.0, 1.0)
    bb2 = BoundingBox1D(0.1, 0.3)
    assert bb2.get_overlap(bb1) == BoundingBox1D(0.1, 0.3)
    assert bb1.get_overlap(bb2) == BoundingBox1D(0.1, 0.3)

    with pytest.raises(TypeError):
        bb1.get_overlap(None)



from tempfile import TemporaryDirectory
import pytest


@pytest.fixture(scope="session", autouse=True)
def changetmp(request):
    with TemporaryDirectory(prefix="pytest-<project-name>-") as temp_dir:
        request.config.option.basetemp = temp_dir
        yield



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2022 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/
__authors__ = ["P. Paleo", "H. Payno"]
__license__ = "MIT"
__date__ = "09/02/2022"


from silx.utils.deprecation import deprecated_warning
from tomoscan.unitsystem.energysystem import (
    EnergySI,
)  # noqa F401  kept for bacward compatibility
from tomoscan.unitsystem.unit import Unit

# Default units:
#  - lenght: meter (m)
#  - energy: kilo Electronvolt (keV)
_meter = 1.0
_kev = 1.0


class MetricSystem(Unit):
    """Util enum to retrieve metric"""

    METER = _meter
    m = _meter
    CENTIMETER = _meter / 100.0
    MILLIMETER = _meter / 1000.0
    MICROMETER = _meter * 1e-6
    NANOMETER = _meter * 1e-9

    KILOELECTRONVOLT = _kev
    ELECTRONVOLT = _kev * 1e-3
    JOULE = _kev / EnergySI.KILOELECTRONVOLT.value
    KILOJOULE = _kev / EnergySI.KILOELECTRONVOLT.value * 1e3

    @classmethod
    def from_str(cls, value: str):
        assert isinstance(value, str)
        if value.lower() in ("m", "meter"):
            return MetricSystem.METER
        elif value.lower() in ("cm", "centimeter"):
            return MetricSystem.CENTIMETER
        elif value.lower() in ("mm", "millimeter"):
            return MetricSystem.MILLIMETER
        elif value.lower() in ("um", "micrometer", "microns"):
            return MetricSystem.MICROMETER
        elif value.lower() in ("nm", "nanometer"):
            deprecated_warning(
                "Function",
                "MetricSystem.from_str for energies",
                reason="Must be part of EnergySI instead",
                replacement="EnergySI.from_str",
                since_version="0.8.0",
            )
            return MetricSystem.NANOMETER
        elif value.lower() in ("kev", "kiloelectronvolt"):
            deprecated_warning(
                "Function",
                "MetricSystem.from_str for energies",
                reason="Must be part of EnergySI instead",
                replacement="EnergySI.from_str",
                since_version="0.8.0",
            )
            return MetricSystem.KILOELECTRONVOLT
        elif value.lower() in ("ev", "electronvolt"):
            deprecated_warning(
                "Function",
                "MetricSystem.from_str for energies",
                reason="Must be part of EnergySI instead",
                replacement="EnergySI.from_str",
                since_version="0.8.0",
            )
            return MetricSystem.ELECTRONVOLT
        elif value.lower() in ("j", "joule"):
            deprecated_warning(
                "Function",
                "MetricSystem.from_str for energies",
                reason="Must be part of EnergySI instead",
                replacement="EnergySI.from_str",
                since_version="0.8.0",
            )
            return MetricSystem.JOULE
        elif value.lower() in ("kj", "kilojoule"):
            deprecated_warning(
                "Function",
                "MetricSystem.from_str for energies",
                reason="Must be part of EnergySI instead",
                replacement="EnergySI.from_str",
                since_version="0.8.0",
            )
            return MetricSystem.KILOJOULE
        else:
            raise ValueError("Cannot convert: %s" % value)

    def __str__(self):
        if self == MetricSystem.METER:
            return "m"
        elif self == MetricSystem.CENTIMETER:
            return "cm"
        elif self == MetricSystem.MILLIMETER:
            return "mm"
        elif self == MetricSystem.MICROMETER:
            return "um"
        elif self == MetricSystem.NANOMETER:
            return "nm"
        elif self == MetricSystem.KILOELECTRONVOLT:
            deprecated_warning(
                "Function",
                "MetricSystem.__str__ for energies",
                reason="Must be part of EnergySI instead",
                replacement="EnergySI.__str__",
                since_version="0.8.0",
            )
            return "keV"
        elif self == MetricSystem.ELECTRONVOLT:
            deprecated_warning(
                "Function",
                "MetricSystem.__str__ for energies",
                reason="Must be part of EnergySI instead",
                replacement="EnergySI.__str__",
                since_version="0.8.0",
            )
            return "eV"
        elif self == MetricSystem.JOULE:
            deprecated_warning(
                "Function",
                "MetricSystem.__str__ for energies",
                reason="Must be part of EnergySI instead",
                replacement="EnergySI.__str__",
                since_version="0.8.0",
            )
            return "J"
        elif self == MetricSystem.KILOJOULE:
            deprecated_warning(
                "Function",
                "MetricSystem.__str__ for energies",
                reason="Must be part of EnergySI instead",
                replacement="EnergySI.__str__",
                since_version="0.8.0",
            )
            return "kJ"
        else:
            raise ValueError(f"Cannot convert: {self}")


m = MetricSystem.METER
meter = MetricSystem.METER

centimeter = MetricSystem.CENTIMETER
cm = centimeter

millimeter = MetricSystem.MILLIMETER
mm = MetricSystem.MILLIMETER

micrometer = MetricSystem.MICROMETER

nanometer = MetricSystem.NANOMETER



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2022 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/
"""utils function for esrf volumes"""


__authors__ = [
    "H. Payno",
]
__license__ = "MIT"
__date__ = "11/07/2022"


import os
import h5py
from tomoscan.esrf.volume.edfvolume import EDFVolume
from tomoscan.esrf.volume.hdf5volume import HDF5Volume
from tomoscan.esrf.volume.tiffvolume import MultiTIFFVolume, TIFFVolume
from tomoscan.esrf.volume.jp2kvolume import JP2KVolume
from tomoscan.esrf.identifier.edfidentifier import EDFVolumeIdentifier
from tomoscan.esrf.identifier.hdf5Identifier import HDF5VolumeIdentifier
from tomoscan.esrf.identifier.tiffidentifier import (
    TIFFVolumeIdentifier,
    MultiTiffVolumeIdentifier,
)
from tomoscan.esrf.identifier.jp2kidentifier import JP2KVolumeIdentifier
from tomoscan.io import HDF5File
from typing import Optional
import logging

_logger = logging.getLogger(__name__)

_DEFAULT_SCHEME_TO_VOL = {
    EDFVolumeIdentifier.scheme: EDFVolume,
    HDF5VolumeIdentifier.scheme: HDF5Volume,
    TIFFVolumeIdentifier.scheme: TIFFVolume,
    MultiTiffVolumeIdentifier.scheme: MultiTIFFVolume,
    JP2KVolumeIdentifier.scheme: JP2KVolume,
}


def guess_hdf5_volume_data_paths(file_path, data_path="/", depth=3) -> tuple:
    """
    browse hdf5 file 'file_path' from 'data_path' on 'depth' level and check for possible defined volumes.

    :param str file_path: file path to the hdf5 file to browse
    :param str data_path: path in the file to start research
    :param int depth: on which layer we should apply research
    :return: tuple of data_path that could fit a volume
    :rtype: tuple
    """
    if not h5py.is_hdf5(file_path):
        raise ValueError(f"{file_path} is not a hdf5 file path")
    with HDF5File(filename=file_path, mode="r") as h5f:
        group = h5f[data_path]
        if isinstance(group, h5py.Group):
            if HDF5Volume.DATA_DATASET_NAME in group:
                return (data_path,)
            elif depth > 0:
                res = []
                for key in group.keys():
                    res.extend(
                        guess_hdf5_volume_data_paths(
                            file_path=file_path,
                            data_path="/".join((data_path, key)).replace("//", "/"),
                            depth=depth - 1,
                        )
                    )
                return tuple(res)

        return tuple()


def guess_volumes(path, scheme_to_vol: Optional[dict] = None) -> tuple:
    """
    from a file path or a folder path try to guess volume(s)

    :param str path: file or folder path
    :param dict scheme_to_vol: dict to know which constructor to call. Key if the scheme, value if the volume constructor.
                               usefull for libraries redefining volume or adding some like tomwer.
                               If none provided will take the tomoscan default one
    :return: tuple of volume
    :rtype: tuple
    """
    if not os.path.exists(path):
        raise OSError("path doesn't exists")

    if scheme_to_vol is None:
        scheme_to_vol = _DEFAULT_SCHEME_TO_VOL

    if os.path.isfile(path):
        if h5py.is_hdf5(path):
            res = []
            for data_path in guess_hdf5_volume_data_paths(path):
                assert isinstance(data_path, str)
                res.append(
                    scheme_to_vol[HDF5VolumeIdentifier.scheme](
                        file_path=path,
                        data_path=data_path,
                    )
                )
            return tuple(res)
        elif path.lower().endswith((".tif", ".tiff")):
            return (scheme_to_vol[MultiTiffVolumeIdentifier.scheme](file_path=path),)
    elif os.path.isdir(path):
        most_common_extension = get_most_common_extension(path)
        if most_common_extension is None:
            return tuple()

        basename = _guess_volume_basename(path, extension=most_common_extension)
        if most_common_extension in ("tiff", "tif"):
            return (
                scheme_to_vol[TIFFVolumeIdentifier.scheme](
                    folder=path,
                    volume_basename=basename,
                    data_extension=most_common_extension,
                ),
            )
        elif most_common_extension in ("jp2", "jp2k"):
            return (
                scheme_to_vol[JP2KVolumeIdentifier.scheme](
                    folder=path,
                    volume_basename=basename,
                    data_extension=most_common_extension,
                ),
            )
        elif most_common_extension == "edf":
            return (
                scheme_to_vol[EDFVolumeIdentifier.scheme](
                    folder=path,
                    volume_basename=basename,
                    data_extension=most_common_extension,
                ),
            )
        else:
            _logger.warning(
                f"most common extension is {most_common_extension}. Unable to create a volume from it"
            )
            return tuple()
    else:
        raise NotImplementedError("guess_volumes only handle file and folder...")


def get_most_common_extension(folder_path):
    if not os.path.isdir(folder_path):
        raise ValueError(f"a folder path is expected. {folder_path} isn't")

    extensions = {}
    for file_path in os.listdir(folder_path):
        _, ext = os.path.splitext(file_path)
        ext = ext.lower().lstrip(".")
        if ext in extensions:
            extensions[ext] += 1
        else:
            extensions[ext] = 1

    # filter not handled extensions
    def is_valid_extension(extension):
        return extension in ("edf", "tif", "tiff", "jp2", "jp2k")

    extensions = {
        key: value for (key, value) in extensions.items() if is_valid_extension(key)
    }

    if len(extensions) == 0:
        _logger.warning(f"no valid extensions found in {folder_path}")
    else:
        sort_extensions = sorted(extensions.items(), key=lambda x: x[1], reverse=True)
        return sort_extensions[0][0]


def _guess_volume_basename(folder_path, extension):
    # list all the files matching the file and guessing the file parttern
    files_to_check = []
    possible_basenames = {}
    for file_path in os.listdir(folder_path):
        if file_path.lower().endswith(extension):
            files_to_check.append(os.path.splitext(file_path)[0])
            # the expected way to save those files is basename_XXXX with XXXX is the index over 4 char
            basename = "_".join(file_path.split("_")[:-1])
            if basename in possible_basenames:
                possible_basenames[basename] += 1
            else:
                possible_basenames[basename] = 1

    if len(possible_basenames) == 0:
        _logger.warning(f"no valid basename found in {folder_path}")
    else:
        sort_basenames = sorted(
            possible_basenames.items(), key=lambda x: x[1], reverse=True
        )
        if len(sort_basenames) > 1:
            _logger.warning(
                f"more than one basename found. Take the most probable one ({sort_basenames[0][0]})"
            )
        return sort_basenames[0][0]



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2022 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/

__authors__ = ["H. Payno"]
__license__ = "MIT"
__date__ = "24/01/2017"


import os
from tomoscan.esrf.edfscan import EDFTomoScan
from tomoscan.esrf.hdf5scan import HDF5TomoScan
from tomoscan.scanbase import TomoScanBase
from tomoscan.factory import Factory
from tomoscan.test.utils import UtilsTest
from tomoscan.esrf.mock import MockEDF
import tempfile
import pytest


def test_scan_edf():
    """can we create a TomoScanBase object from a folder containing a
    valid .edf acquisition"""
    scan_dir = UtilsTest.getDataset("test10")
    scan = Factory.create_scan_object(scan_dir)
    assert isinstance(scan, EDFTomoScan)


def test_one_nx():
    """Can we create a TomoScanBase from a .nx master file containing
    one acquisition"""
    file_name = "frm_edftomomill_oneentry.nx"
    master_file = UtilsTest.getH5Dataset(file_name)
    scan = Factory.create_scan_object(master_file)
    assert isinstance(scan, HDF5TomoScan)
    assert scan.path == os.path.dirname(master_file)
    assert scan.master_file == master_file
    assert scan.entry == "entry"


def test_one_two_nx():
    """Can we create a TomoScanBase from a .nx master file containing
    two acquisitions"""
    master_file = UtilsTest.getH5Dataset("frm_edftomomill_twoentries.nx")
    scan = Factory.create_scan_object(master_file)
    assert isinstance(scan, HDF5TomoScan)
    assert scan.path == os.path.dirname(master_file)
    assert scan.master_file == master_file
    assert scan.entry == "entry0000"


def test_two_nx():
    """Can we create two TomoScanBase from a .nx master file containing
    two acquisitions using the Factory"""
    master_file = UtilsTest.getH5Dataset("frm_edftomomill_twoentries.nx")
    scans = Factory.create_scan_objects(master_file)
    assert len(scans) == 2
    for scan, scan_entry in zip(scans, ("entry0000", "entry0001")):
        assert isinstance(scan, HDF5TomoScan) is True
        assert scan.path == os.path.dirname(master_file)
        assert scan.master_file == master_file
        assert scan.entry == scan_entry


def test_invalid_path():
    """Insure an error is raised if the path as no meaning"""
    with pytest.raises(ValueError):
        Factory.create_scan_object("toto")

    with pytest.raises(ValueError):
        Factory.create_scan_objects("toto")

    with tempfile.TemporaryDirectory() as scan_dir:
        with pytest.raises(ValueError):
            Factory.create_scan_object(scan_dir)


def test_edf_scan_creation():
    with tempfile.TemporaryDirectory() as folder:
        scan_dir = os.path.join(folder, "my_scan")
        MockEDF.mockScan(scanID=scan_dir, nRecons=10)
        scan = Factory.create_scan_object(scan_path=scan_dir)
        assert isinstance(scan, EDFTomoScan)
        scans = Factory.create_scan_objects(scan_path=scan_dir)
        assert len(scans) == 1
        assert isinstance(scans[0], EDFTomoScan)
        dict_ = scan.to_dict()
        Factory.create_scan_object_frm_dict(dict_)
        # test invalid dict
        dict_[TomoScanBase.DICT_TYPE_KEY] = "tata"
        with pytest.raises(ValueError):
            Factory.create_scan_object_frm_dict(dict_)
        del dict_[TomoScanBase.DICT_TYPE_KEY]
        with pytest.raises(ValueError):
            Factory.create_scan_object_frm_dict(dict_)



class NEXUS_MONITOR_PATH:

    DATA_PATH = "data"


class NEXUS_MONITOR_PATH_V_1_0(NEXUS_MONITOR_PATH):
    pass


class NEXUS_MONITOR_PATH_V_1_1(NEXUS_MONITOR_PATH_V_1_0):
    pass


class NEXUS_MONITOR_PATH_V_1_2(NEXUS_MONITOR_PATH_V_1_1):
    pass



# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2022 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/
"""module for giving information on process progress"""

__authors__ = ["H. Payno"]
__license__ = "MIT"
__date__ = "31/08/2021"


import tomoscan.progress


def test_progress():
    """Simple test of the Progress API"""
    progress = tomoscan.progress.Progress("this is progress")
    progress.reset()
    progress.startProcess()
    progress.setMaxAdvancement(80)
    for adv in (10, 20, 50, 70):
        progress.setAdvancement(adv)
    for i in range(10):
        progress.increaseAdvancement(1)


def test_advancement():
    """Simple test of the _Advancement API"""
    for i in range(4):
        tomoscan.progress._Advancement.getNextStep(
            tomoscan.progress._Advancement.getStep(i)
        )
